@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: 33bf1c5a238b97e090487defbb1df167770a0f38 $
+# $Id: 2a4d3de44b06eae6da137811e23e94d9067c4702 $
 
 longusage() {
   echo "Gentoo Linux Genkernel ${GK_V}"
@@ -86,20 +86,36 @@ longusage() {
   echo "  Low-Level Compile settings"
   echo "	--cross-compile=<target-triplet>"
   echo "				Target triple (i.e. aarch64-linux-gnu) to build for"
+  echo "	--kernel-cross-compile=<target-triplet>"
+  echo "				Target triple (i.e. aarch64-linux-gnu) to build kernel for"
+  echo "	--kernel-ar=<archiver>	Archiver to use for kernel"
   echo "	--kernel-as=<assembler>	Assembler to use for kernel"
   echo "	--kernel-cc=<compiler>	Compiler to use for kernel (e.g. distcc)"
   echo "	--kernel-ld=<linker>	Linker to use for kernel"
   echo "	--kernel-make=<makeprg> GNU Make to use for kernel"
+  echo "	--kernel-nm=<nm>	NM utility to use for kernel"
+  echo "	--kernel-objcopy=<objcopy>	OBJCOPY utility to use for kernel"
+  echo "	--kernel-objdump=<objdump>	OBJDUMP utility to use for kernel"
+  echo "	--kernel-ranlib=<ranlib>	RANLIB utility to use for kernel"
+  echo "	--kernel-readelf=<readelf>	READELF utility to use for kernel"
+  echo "	--kernel-strip=<strip>	STRIP utility to use for kernel"
   echo "	--kernel-target=<t>	Override default make target (bzImage)"
   echo "	--kernel-binary=<path>	Override default kernel binary path (arch/foo/boot/bar)"
   echo "	--kernel-outputdir=<path>"
   echo "				Save output files outside the source tree"
+  echo "	--utils-ar=<archiver>	Archiver to use for utils"
   echo "	--utils-as=<assembler>	Assembler to use for utils"
   echo "	--utils-cc=<compiler>	C Compiler to use for utilities"
   echo "	--utils-cxx=<compiler>	C++ Compiler to use for utilities"
   echo "	--utils-cflags=<cflags> C compiler flags used to compile utilities"
   echo "	--utils-ld=<linker>	Linker to use for utils"
   echo "	--utils-make=<makeprog>	GNU Make to use for utils"
+  echo "	--utils-nm=<nm>		NM utility to use for utils"
+  echo "	--utils-objcopy=<objcopy>	OBJCOPY utility to use for utils"
+  echo "	--utils-objdump=<objdump>	OBJDUMP utility to use for utils"
+  echo "	--utils-ranlib=<ranlib>		RANLIB utility to use for utils"
+  echo "	--utils-readelf=<readelf>	READELF utility to use for utils"
+  echo "	--utils-strip=<strip>	STRIP utility to use for utils"
   echo "	--makeopts=<makeopts>	Make options such as -j2, etc ..."
   echo "	--mountboot		Mount BOOTDIR automatically if mountable"
   echo "	--no-mountboot		Don't mount BOOTDIR automatically"
@@ -167,6 +183,8 @@ longusage() {
   echo "	--no-luks		Exclude LUKS support"
   echo "	--gpg			Include GPG-armored LUKS key support"
   echo "	--no-gpg		Exclude GPG-armored LUKS key support"
+  echo "	--keyctl		Include keyctl support for loading LUKS passphrase into a keyring"
+  echo "	--no-keyctl		Exclude keyctl support for loading LUKS passphrase into a keyring"
   echo "	--b2sum			Include b2sum"
   echo "	--no-b2sum		Exclude b2sum"
   echo "	--busybox		Include busybox"
@@ -288,6 +306,18 @@ parse_cmdline() {
 			CMD_CROSS_COMPILE="${*#*=}"
 			print_info 3 "CMD_CROSS_COMPILE: ${CMD_CROSS_COMPILE}"
 			;;
+			--kernel-cross-compile=*)
+			CMD_KERNEL_CROSS_COMPILE="${*#*=}"
+			print_info 3 "CMD_KERNEL_CROSS_COMPILE: ${CMD_KERNEL_CROSS_COMPILE}"
+			;;
+		--kernel-ar=*)
+			CMD_KERNEL_AR="${*#*=}"
+			print_info 3 "CMD_KERNEL_AR: ${CMD_KERNEL_AR}"
+			;;
+		--kernel-as=*)
+			CMD_KERNEL_AS="${*#*=}"
+			print_info 3 "CMD_KERNEL_AS: ${CMD_KERNEL_AS}"
+			;;
 		--kernel-cc=*)
 			CMD_KERNEL_CC="${*#*=}"
 			print_info 3 "CMD_KERNEL_CC: ${CMD_KERNEL_CC}"
@@ -296,14 +326,34 @@ parse_cmdline() {
 			CMD_KERNEL_LD="${*#*=}"
 			print_info 3 "CMD_KERNEL_LD: ${CMD_KERNEL_LD}"
 			;;
-		--kernel-as=*)
-			CMD_KERNEL_AS="${*#*=}"
-			print_info 3 "CMD_KERNEL_AS: ${CMD_KERNEL_AS}"
-			;;
 		--kernel-make=*)
 			CMD_KERNEL_MAKE="${*#*=}"
 			print_info 3 "CMD_KERNEL_MAKE: ${CMD_KERNEL_MAKE}"
 			;;
+		--kernel-nm=*)
+			CMD_KERNEL_NM="${*#*=}"
+			print_info 3 "CMD_KERNEL_NM: ${CMD_KERNEL_NM}"
+			;;
+		--kernel-objcopy=*)
+			CMD_KERNEL_OBJCOPY="${*#*=}"
+			print_info 3 "CMD_KERNEL_OBJCOPY: ${CMD_KERNEL_OBJCOPY}"
+			;;
+		--kernel-objdump=*)
+			CMD_KERNEL_OBJDUMP="${*#*=}"
+			print_info 3 "CMD_KERNEL_OBJDUMP: ${CMD_KERNEL_OBJDUMP}"
+			;;
+		--kernel-ranlib=*)
+			CMD_KERNEL_RANLIB="${*#*=}"
+			print_info 3 "CMD_KERNEL_RANLIB: ${CMD_KERNEL_RANLIB}"
+			;;
+		--kernel-readelf=*)
+			CMD_KERNEL_READELF="${*#*=}"
+			print_info 3 "CMD_KERNEL_READELF: ${CMD_KERNEL_READELF}"
+			;;
+		--kernel-strip=*)
+			CMD_KERNEL_STRIP="${*#*=}"
+			print_info 3 "CMD_KERNEL_STRIP: ${CMD_KERNEL_STRIP}"
+			;;
 		--kernel-target=*)
 			KERNEL_MAKE_DIRECTIVE_OVERRIDE="${*#*=}"
 			print_info 3 "KERNEL_MAKE_DIRECTIVE_OVERRIDE: ${KERNEL_MAKE_DIRECTIVE_OVERRIDE}"
@@ -316,6 +366,14 @@ parse_cmdline() {
 			CMD_KERNEL_OUTPUTDIR="${*#*=}"
 			print_info 3 "CMD_KERNEL_OUTPUTDIR: ${CMD_KERNEL_OUTPUTDIR}"
 			;;
+		--utils-ar=*)
+			CMD_UTILS_AR="${*#*=}"
+			print_info 3 "CMD_UTILS_AR: ${CMD_UTILS_AR}"
+			;;
+		--utils-as=*)
+			CMD_UTILS_AS="${*#*=}"
+			print_info 3 "CMD_UTILS_AS: ${CMD_UTILS_AS}"
+			;;
 		--utils-cc=*)
 			CMD_UTILS_CC="${*#*=}"
 			print_info 3 "CMD_UTILS_CC: ${CMD_UTILS_CC}"
@@ -332,14 +390,34 @@ parse_cmdline() {
 			CMD_UTILS_LD="${*#*=}"
 			print_info 3 "CMD_UTILS_LD: ${CMD_UTILS_LD}"
 			;;
-		--utils-as=*)
-			CMD_UTILS_AS="${*#*=}"
-			print_info 3 "CMD_UTILS_AS: ${CMD_UTILS_AS}"
-			;;
 		--utils-make=*)
 			CMD_UTILS_MAKE="${*#*=}"
 			print_info 3 "CMD_UTILS_MAKE: ${CMD_UTILS_MAKE}"
 			;;
+		--utils-nm=*)
+			CMD_UTILS_NM="${*#*=}"
+			print_info 3 "CMD_UTILS_NM: ${CMD_UTILS_NM}"
+			;;
+		--utils-objcopy=*)
+			CMD_UTILS_OBJCOPY="${*#*=}"
+			print_info 3 "CMD_UTILS_OBJCOPY: ${CMD_UTILS_OBJCOPY}"
+			;;
+		--utils-objdump=*)
+			CMD_UTILS_OBJDUMP="${*#*=}"
+			print_info 3 "CMD_UTILS_OBJDUMP: ${CMD_UTILS_OBJDUMP}"
+			;;
+		--utils-ranlib=*)
+			CMD_UTILS_RANLIB="${*#*=}"
+			print_info 3 "CMD_UTILS_RANLIB: ${CMD_UTILS_RANLIB}"
+			;;
+		--utils-readelf=*)
+			CMD_UTILS_READELF="${*#*=}"
+			print_info 3 "CMD_UTILS_READELF: ${CMD_UTILS_READELF}"
+			;;
+		--utils-strip=*)
+			CMD_UTILS_STRIP="${*#*=}"
+			print_info 3 "CMD_UTILS_STRIP: ${CMD_UTILS_STRIP}"
+			;;
 		--makeopts=*)
 			CMD_MAKEOPTS="${*#*=}"
 			print_info 3 "CMD_MAKEOPTS: ${CMD_MAKEOPTS}"
@@ -767,6 +845,10 @@ parse_cmdline() {
 			CMD_GPG=$(parse_optbool "$*")
 			print_info 3 "CMD_GPG: ${CMD_GPG}"
 			;;
+		--keyctl|--no-keyctl)
+			CMD_KEYCTL=$(parse_optbool "$*")
+			print_info 3 "CMD_KEYCTL: ${CMD_KEYCTL}"
+			;;
 		--firmware|--no-firmware)
 			CMD_FIRMWARE=$(parse_optbool "$*")
 			print_info 3 "CMD_FIRMWARE: ${CMD_FIRMWARE}"
