@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: e5254d779350b757134a2d7b5d4d5990036ec572 $
+# $Id: 9af128b912c235c4f0849f7cc182ab89398ebcd2 $
 
 compile_external_modules() {
 	if ! isTrue "${CMD_MODULEREBUILD}"
@@ -20,17 +20,10 @@ compile_external_modules() {
 		return
 	fi
 
-	if [ -n "${INSTALL_MOD_PATH}" ]
+	if [ -n "${KERNEL_MODULES_PREFIX}" ]
 	then
 		# emerge would install to a different location
-		print_warning 1 "$(get_indent 1)>> INSTALL_MOD_PATH set; Skipping '${MODULEREBUILD_CMD}' ..."
-		return
-	fi
-
-	local modulesdb_file="/var/lib/module-rebuild/moduledb"
-	if [ ! -s "${modulesdb_file}" ]
-	then
-		print_info 2 "$(get_indent 1)>> '${modulesdb_file}' does not exist or is empty; Skipping '${MODULEREBUILD_CMD}' ..."
+		print_warning 1 "$(get_indent 1)>> KERNEL_MODULES_PREFIX set; Skipping '${MODULEREBUILD_CMD}' ..."
 		return
 	fi
 
@@ -80,86 +73,43 @@ compile_generic() {
 	local argstype=${2}
 	local RET
 
+	local -a compile_cmd=()
+
+	if [ ${NICE} -ne 0 ]
+	then
+		compile_cmd+=( nice "-n${NICE}" )
+	fi
+
 	case "${argstype}" in
 		kernel|kernelruntask)
 			if [ -z "${KERNEL_MAKE}" ]
 			then
 				gen_die "KERNEL_MAKE undefined - I don't know how to compile a kernel for this arch!"
 			else
-				local MAKE=${KERNEL_MAKE}
+				local -x MAKE=${KERNEL_MAKE}
+				compile_cmd+=( "${MAKE}" "${MAKEOPTS}" )
 			fi
 
-			# Build kernel compile parameter.
-			local ARGS=""
-
-			# Allow for CC/LD... user override!
-			local -a kernel_vars
-			kernel_vars+=( 'ARCH' )
-			kernel_vars+=( 'AS' )
-			kernel_vars+=( 'CC' )
-			kernel_vars+=( 'LD' )
-
-			local kernel_var=
-			for kernel_var in "${kernel_vars[@]}"
-			do
-				local kernel_varname="KERNEL_${kernel_var}"
-				local kernel_default_varname="DEFAULT_${kernel_varname}"
-
-				if [[ -z "${!kernel_default_varname}" ]] \
-					|| [[ -n "${!kernel_default_varname}" ]] \
-					&& [[ "${!kernel_varname}" != "${!kernel_default_varname}" ]]
-				then
-					ARGS="${ARGS} ${kernel_var}=\"${!kernel_varname}\""
-				fi
-			done
-			unset kernel_var kernel_vars kernel_varname kernel_default_varname
-
-			if isTrue "$(tc-is-cross-compiler)"
+			if [[ "${argstype}" == 'kernelruntask' ]]
 			then
-				local can_tc_cross_compile=no
-				local cpu_cbuild=${CBUILD%%-*}
-				local cpu_chost=${CHOST%%-*}
-
-				case "${cpu_cbuild}" in
-					powerpc64*)
-						if [[ "${cpu_chost}" == "powerpc" ]]
-						then
-							can_tc_cross_compile=yes
-						fi
-						;;
-					x86_64*)
-						if [[ "${cpu_chost}" == "i686" ]]
-						then
-							can_tc_cross_compile=yes
-						fi
-						;;
-				esac
-
-				if isTrue "${can_tc_cross_compile}"
-				then
-					local -a kernel_vars
-					kernel_vars+=( 'AS' )
-					kernel_vars+=( 'CC' )
-					kernel_vars+=( 'LD' )
-
-					local kernel_var=
-					for kernel_var in "${kernel_vars[@]}"
-					do
-						if [[ "${ARGS}" == *${kernel_var}=* ]]
-						then
-							# User wants to run specific program ...
-							continue
-						else
-							ARGS="${ARGS} ${kernel_var}=\"$(tc-get${kernel_var})\""
-						fi
-					done
-					unset kernel_var kernel_vars
-				else
-					ARGS="${ARGS} CROSS_COMPILE=\"${CHOST}-\""
-				fi
-				unset can_tc_cross_compile cpu_cbuild cpu_chost
+				# silent operation, forced -j1
+				compile_cmd+=( -s -j1 )
 			fi
 
+			# Pass kernel compile parameter
+			compile_cmd+=( "ARCH='${KERNEL_ARCH}'" )
+
+			local tc_var
+			for tc_var in AS AR CC LD NM OBJCOPY OBJDUMP READELF STRIP
+			do
+				compile_cmd+=( "${tc_var}='$(TC_PROG_TYPE=KERNEL tc-get${tc_var})'" )
+			done
+
+			compile_cmd+=( "HOSTAR='$(tc-getBUILD_AR)'" )
+			compile_cmd+=( "HOSTCC='$(tc-getBUILD_CC)'" )
+			compile_cmd+=( "HOSTCXX='$(tc-getBUILD_CXX)'" )
+			compile_cmd+=( "HOSTLD='$(tc-getBUILD_LD)'" )
+
 			if [ -n "${KERNEL_OUTPUTDIR}" -a "${KERNEL_OUTPUTDIR}" != "${KERNEL_DIR}" ]
 			then
 				if [ -f "${KERNEL_DIR}/.config" -o -d "${KERNEL_DIR}/include/config" ]
@@ -174,7 +124,7 @@ compile_generic() {
 					error_message+=" Please re-install a fresh kernel source!"
 					gen_die "${error_message}"
 				else
-					ARGS="${ARGS} O=\"${KERNEL_OUTPUTDIR}\""
+					compile_cmd+=( "O='${KERNEL_OUTPUTDIR}'" )
 				fi
 			fi
 			;;
@@ -184,34 +134,30 @@ compile_generic() {
 			gen_die "${error_msg}"
 			;;
 	esac
-	shift 2
 
-	if [ ${NICE} -ne 0 ]
-	then
-		NICEOPTS="nice -n${NICE} "
-	else
-		NICEOPTS=""
-	fi
+	compile_cmd+=( "${target}" )
+
+	print_info 3 "COMMAND: ${compile_cmd[*]}" 1 0 1
 
 	# the eval usage is needed in the next set of code
 	# as ARGS can contain spaces and quotes, eg:
 	# ARGS='CC="ccache gcc"'
-	if [ "${argstype}" == 'kernelruntask' ]
+	if [[ "${argstype}" == 'kernelruntask' ]]
 	then
-		# Silent operation, forced -j1
-		print_info 3 "COMMAND: ${NICEOPTS}${MAKE} ${MAKEOPTS} -j1 ${ARGS} ${target} $*" 1 0 1
-		eval ${NICEOPTS}${MAKE} -s ${MAKEOPTS} -j1 ${ARGS} ${target} $*
+		eval "${compile_cmd[@]}"
 		RET=$?
 	elif [ "${LOGLEVEL}" -gt 3 ]
 	then
 		# Output to stdout and logfile
-		print_info 3 "COMMAND: ${NICEOPTS}${MAKE} ${MAKEOPTS} ${ARGS} ${target} $*" 1 0 1
-		eval ${NICEOPTS}${MAKE} ${MAKEOPTS} ${ARGS} ${target} $* 2>&1 | tee -a "${LOGFILE}"
+		compile_cmd+=( "2>&1 | tee -a '${LOGFILE}'" )
+
+		eval "${compile_cmd[@]}"
 		RET=${PIPESTATUS[0]}
 	else
 		# Output to logfile only
-		print_info 3 "COMMAND: ${NICEOPTS}${MAKE} ${MAKEOPTS} ${ARGS} ${target} $*" 1 0 1
-		eval ${NICEOPTS}${MAKE} ${MAKEOPTS} ${ARGS} ${target} $* >> "${LOGFILE}" 2>&1
+		compile_cmd+=( ">> '${LOGFILE}' 2>&1" )
+
+		eval "${compile_cmd[@]}"
 		RET=$?
 	fi
 
@@ -229,7 +175,7 @@ compile_modules() {
 
 	compile_generic modules kernel
 
-	[ -n "${INSTALL_MOD_PATH}" ] && local -x INSTALL_MOD_PATH="${INSTALL_MOD_PATH}"
+	[ -n "${KERNEL_MODULES_PREFIX}" ] && local -x INSTALL_MOD_PATH="${KERNEL_MODULES_PREFIX%/}"
 	if [ "${CMD_STRIP_TYPE}" == "all" -o "${CMD_STRIP_TYPE}" == "modules" ]
 	then
 		print_info 1 "$(get_indent 1)>> Installing ${KV} modules (and stripping) ..."
@@ -242,10 +188,10 @@ compile_modules() {
 	compile_generic "modules_install" kernel
 
 	print_info 1 "$(get_indent 1)>> Generating module dependency data ..."
-	if [ -n "${INSTALL_MOD_PATH}" ]
+	if [ -n "${KERNEL_MODULES_PREFIX}" ]
 	then
-		depmod -a -e -F "${KERNEL_OUTPUTDIR}"/System.map -b "${INSTALL_MOD_PATH}" ${KV} \
-			|| gen_die "depmod (INSTALL_MOD_PATH=${INSTALL_MOD_PATH}) failed!"
+		depmod -a -e -F "${KERNEL_OUTPUTDIR}"/System.map -b "${KERNEL_MODULES_PREFIX%/}" ${KV} \
+			|| gen_die "depmod (INSTALL_MOD_PATH=${KERNEL_MODULES_PREFIX%/}) failed!"
 	else
 		depmod -a -e -F "${KERNEL_OUTPUTDIR}"/System.map ${KV} \
 			|| gen_die "depmod failed!"
@@ -297,8 +243,8 @@ compile_kernel() {
 			print_info 1 "$(get_indent 1)>> Not installing firmware as it's included in the kernel already (CONFIG_FIRMWARE_IN_KERNEL=y) ..."
 		else
 			print_info 1 "$(get_indent 1)>> Installing firmware ('make firmware_install') due to CONFIG_FIRMWARE_IN_KERNEL != y ..."
-			[ "${INSTALL_MOD_PATH}" != '' ] && export INSTALL_MOD_PATH
-			[ "${INSTALL_FW_PATH}" != '' ] && export INSTALL_FW_PATH
+			[ -n "${KERNEL_MODULES_PREFIX}" ] && local -x INSTALL_MOD_PATH="${KERNEL_MODULES_PREFIX%/}"
+			[ -n "${INSTALL_FW_PATH}" ] && export INSTALL_FW_PATH
 			MAKEOPTS="${MAKEOPTS} -j1" compile_generic "firmware_install" kernel
 		fi
 	elif [ ${KV_NUMERIC} -lt 4014 ]
@@ -557,6 +503,28 @@ populate_binpkg() {
 		unset patch patchdir
 	fi
 
+	if [[ -f "${BINPKG}" ]]
+	then
+		if isTrue "$(is_glibc)"
+		then
+			local libdir=$(get_chost_libdir)
+			local glibc_test_file="${libdir}/libc.a"
+
+			if [[ ! -f "${glibc_test_file}" ]]
+			then
+				gen_die "Internal error: File '${glibc_test_file}' not found! Do not know how to deal with this glibc version."
+			fi
+
+			if [[ "${BINPKG}" -ot "${glibc_test_file}" ]]
+			then
+				print_info 3 "${CHECK_LEVEL_PREFIX}Glibc (${glibc_test_file}) is newer than us; Removing stale ${P} binpkg ..."
+				rm "${BINPKG}" || gen_die "Failed to remove stale binpkg '${BINPKG}'!"
+			fi
+
+			print_info 3 "${CHECK_LEVEL_PREFIX}Existing ${P} binpkg is newer than glibc (${glibc_test_file}); Skipping ..."
+		fi
+	fi
+
 	if [[ ! -f "${BINPKG}" ]]
 	then
 		print_info 3 "${CHECK_LEVEL_PREFIX}Binpkg '${BINPKG}' does NOT exist; Need to build ${P} ..."
