@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: 56b566502ba88e467e847e36e04cd626be12b519 $
+# $Id: 62113dca1ad438c607ca5f75b493087d41d4ede3 $
 
 # Fills variable KERNEL_CONFIG
 determine_kernel_config_file() {
@@ -231,7 +231,7 @@ set_initramfs_compression_method() {
 }
 
 config_kernel() {
-	local diff_cmd="$(which zdiff 2>/dev/null)"
+	local diff_cmd="$(type -P zdiff 2>/dev/null)"
 	if [ -z "${diff_cmd}" ]
 	then
 		print_warning 5 "zdiff is not available."
@@ -334,8 +334,24 @@ config_kernel() {
 
 	if [ -n "${add_config}" ]
 	then
+		local kconfig_md5sum_old="$(md5sum < "${KERNEL_OUTPUTDIR}/.config")"
+
 		print_info 1 "$(get_indent 1)>> Invoking ${add_config} ..."
 		compile_generic ${add_config} kernelruntask
+
+		local kconfig_md5sum_new="$(md5sum < "${KERNEL_OUTPUTDIR}/.config")"
+		if [[ "${kconfig_md5sum_old}" == "${kconfig_md5sum_new}" ]]
+		then
+			print_warning 1 "$(get_indent 1)>> Kernel config was not modified!"
+		fi
+		unset kconfig_md5sum_old kconfig_md5sum_new
+	fi
+
+	# Check for suitable kmod
+	determine_KEXT
+	if ! isTrue "$(is_kext_supported_by_kmod "${KEXT}")"
+	then
+		gen_die "${KMOD_CMD} does not support chosen module compression algorithm. Please re-emerge sys-apps/kmod with USE=$(get_kext_kmod_use_flag "${KEXT}") enabled or adjust CONFIG_MODULE_COMPRESS_* kernel option!"
 	fi
 
 	local -a required_kernel_options
@@ -978,7 +994,7 @@ config_kernel() {
 
 			print_info 2 "$(get_indent 1)>> Ensure that required kernel options for early microcode loading support are set ..."
 			kconfigs_microcode+=( 'CONFIG_MICROCODE' )
-			kconfigs_microcode+=( 'CONFIG_MICROCODE_OLD_INTERFACE' )
+			[ ${KV_NUMERIC} -le 4003 ] && kconfigs_microcode+=( 'CONFIG_MICROCODE_OLD_INTERFACE' )
 			[ ${KV_NUMERIC} -le 4003 ] && kconfigs_microcode+=( 'CONFIG_MICROCODE_EARLY' )
 
 			# Intel
