@@ -1,5 +1,40 @@
 #!/bin/bash
-# $Id: f28c8d65487798d837ff734f25b9fca2c71a6cde $
+# $Id: 7122ea3d3051138f03b846512091d30c87d89b0c $
+
+determine_KEXT() {
+	local kconfig_file_used="${KERNEL_CONFIG}"
+	if isTrue "${BUILD_KERNEL}"
+	then
+		kconfig_file_used="${KERNEL_OUTPUTDIR}/.config"
+	fi
+
+	KEXT='.ko'
+
+	declare -A module_compression_algorithms=()
+	module_compression_algorithms[NONE]='.ko'
+	module_compression_algorithms[GZIP]='.ko.gz'
+	module_compression_algorithms[XZ]='.ko.xz'
+	module_compression_algorithms[ZSTD]='.ko.zst'
+
+	local module_compression_algorithm
+	for module_compression_algorithm in "${!module_compression_algorithms[@]}"
+	do
+		print_info 5 "Checking if module compression algorithm '${module_compression_algorithm}' is being used ..."
+
+		local koption="CONFIG_MODULE_COMPRESS_${module_compression_algorithm}"
+		local value_koption=$(kconfig_get_opt "${kconfig_file_used}" "${koption}")
+		if [[ "${value_koption}" != "y" ]]
+		then
+			print_info 5 "Cannot use '${module_compression_algorithm}' algorithm for module compression, kernel option '${koption}' is not set!"
+			continue
+		fi
+
+		print_info 5 "Will use '${module_compression_algorithm}' algorithm for kernel module compression!"
+		KEXT="${module_compression_algorithms[${module_compression_algorithm}]}"
+		break
+	done
+	unset module_compression_algorithms module_compression_algorithm koption value_koption
+}
 
 determine_KV() {
 	local old_KV=
@@ -318,19 +353,20 @@ determine_real_args() {
 	set_config_with_override STRING CHOST                                 CMD_CHOST                                 "${DEFAULT_CHOST}"
 	set_config_with_override STRING MAKEOPTS                              CMD_MAKEOPTS                              "${DEFAULT_MAKEOPTS}"
 	set_config_with_override STRING NICE                                  CMD_NICE                                  "10"
-	set_config_with_override STRING KERNEL_AS                             CMD_KERNEL_AS                             "${DEFAULT_KERNEL_AS}"
-	set_config_with_override STRING KERNEL_AR                             CMD_KERNEL_AR                             "${DEFAULT_KERNEL_AR}"
-	set_config_with_override STRING KERNEL_CC                             CMD_KERNEL_CC                             "${DEFAULT_KERNEL_CC}"
-	set_config_with_override STRING KERNEL_CFLAGS                         CMD_KERNEL_CFLAGS                         "${DEFAULT_KERNEL_CFLAGS}"
+	set_config_with_override STRING KERNEL_CROSS_COMPILE                  CMD_KERNEL_CROSS_COMPILE
+	set_config_with_override STRING KERNEL_AS                             CMD_KERNEL_AS                             "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_AS}" || echo "${DEFAULT_KERNEL_AS}")"
+	set_config_with_override STRING KERNEL_AR                             CMD_KERNEL_AR                             "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_AR}" || echo "${DEFAULT_KERNEL_AR}")"
+	set_config_with_override STRING KERNEL_CC                             CMD_KERNEL_CC                             "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_CC}" || echo "${DEFAULT_KERNEL_CC}")"
 	set_config_with_override STRING KERNEL_CXX                            CMD_KERNEL_CXX                            "${DEFAULT_KERNEL_CXX}"
-	set_config_with_override STRING KERNEL_LD                             CMD_KERNEL_LD                             "${DEFAULT_KERNEL_LD}"
-	set_config_with_override STRING KERNEL_NM                             CMD_KERNEL_NM                             "${DEFAULT_KERNEL_NM}"
+	set_config_with_override STRING KERNEL_CFLAGS                         CMD_KERNEL_CFLAGS                         "${DEFAULT_KERNEL_CFLAGS}"
+	set_config_with_override STRING KERNEL_LD                             CMD_KERNEL_LD                             "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_LD}" || echo "${DEFAULT_KERNEL_LD}")"
 	set_config_with_override STRING KERNEL_MAKE                           CMD_KERNEL_MAKE                           "${DEFAULT_KERNEL_MAKE}"
-	set_config_with_override STRING KERNEL_OBJCOPY                        CMD_KERNEL_OBJCOPY                        "${DEFAULT_KERNEL_OBJCOPY}"
-	set_config_with_override STRING KERNEL_OBJDUMP                        CMD_KERNEL_OBJDUMP                        "${DEFAULT_KERNEL_OBJDUMP}"
-	set_config_with_override STRING KERNEL_RANLIB                         CMD_KERNEL_RANLIB                         "${DEFAULT_KERNEL_RANLIB}"
-	set_config_with_override STRING KERNEL_READELF                        CMD_KERNEL_READELF                        "${DEFAULT_KERNEL_READELF}"
-	set_config_with_override STRING KERNEL_STRIP                          CMD_KERNEL_STRIP                          "${DEFAULT_KERNEL_STRIP}"
+	set_config_with_override STRING KERNEL_NM                             CMD_KERNEL_NM                             "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_NM}" || echo "${DEFAULT_KERNEL_NM}")"
+	set_config_with_override STRING KERNEL_OBJCOPY                        CMD_KERNEL_OBJCOPY                        "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_OBJCOPY}" || echo "${DEFAULT_KERNEL_OBJCOPY}")"
+	set_config_with_override STRING KERNEL_OBJDUMP                        CMD_KERNEL_OBJDUMP                        "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_OBJDUMP}" || echo "${DEFAULT_KERNEL_OBJDUMP}")"
+	set_config_with_override STRING KERNEL_RANLIB                         CMD_KERNEL_RANLIB                         "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_RANLIB}" || echo "${DEFAULT_KERNEL_RANLIB}")"
+	set_config_with_override STRING KERNEL_READELF                        CMD_KERNEL_READELF                        "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_READELF}" || echo "${DEFAULT_KERNEL_READELF}")"
+	set_config_with_override STRING KERNEL_STRIP                          CMD_KERNEL_STRIP                          "$([[ -n "${KERNEL_CROSS_COMPILE}" ]] && echo "${KERNEL_CROSS_COMPILE}-${DEFAULT_KERNEL_STRIP}" || echo "${DEFAULT_KERNEL_STRIP}")"
 	set_config_with_override STRING UTILS_AS                              CMD_UTILS_AS                              "${DEFAULT_UTILS_AS}"
 	set_config_with_override STRING UTILS_AR                              CMD_UTILS_AR                              "${DEFAULT_UTILS_AR}"
 	set_config_with_override STRING UTILS_CC                              CMD_UTILS_CC                              "${DEFAULT_UTILS_CC}"
@@ -398,6 +434,7 @@ determine_real_args() {
 	set_config_with_override STRING REAL_ROOT                             CMD_REAL_ROOT
 	set_config_with_override BOOL   LUKS                                  CMD_LUKS                                  "no"
 	set_config_with_override BOOL   GPG                                   CMD_GPG                                   "no"
+	set_config_with_override BOOL   KEYCTL                                CMD_KEYCTL                                "no"
 	set_config_with_override BOOL   MDADM                                 CMD_MDADM                                 "no"
 	set_config_with_override STRING MDADM_CONFIG                          CMD_MDADM_CONFIG
 	set_config_with_override BOOL   E2FSPROGS                             CMD_E2FSPROGS                             "no"
@@ -415,7 +452,6 @@ determine_real_args() {
 	set_config_with_override BOOL   GENZIMAGE                             CMD_GENZIMAGE                             "no"
 	set_config_with_override BOOL   KEYMAP                                CMD_KEYMAP                                "yes"
 	set_config_with_override BOOL   DOKEYMAPAUTO                          CMD_DOKEYMAPAUTO                          "no"
-	set_config_with_override STRING BUSYBOX_CONFIG                        CMD_BUSYBOX_CONFIG
 	set_config_with_override STRING STRIP_TYPE                            CMD_STRIP_TYPE                            "modules"
 	set_config_with_override BOOL   INSTALL                               CMD_INSTALL                               "yes"
 	set_config_with_override BOOL   CLEANUP                               CMD_CLEANUP                               "yes"
@@ -934,7 +970,7 @@ determine_real_args() {
 		then
 			if [[ "${CMD_BOOTFONT}" == "current" ]]
 			then
-				SETFONT_COMMAND="$(which setfont 2>/dev/null)"
+				SETFONT_COMMAND="$(type -P setfont 2>/dev/null)"
 				if [ -z "${SETFONT_COMMAND}" ]
 				then
 					gen_die "setfont not found. Is sys-apps/kbd installed?"
@@ -1043,7 +1079,7 @@ determine_real_args() {
 			if [ ! -x "/lib/udev/scsi_id" ]
 			then
 				local error_msg="'/lib/udev/scsi_id' is required for --multipath but file does not exist or is not executable!"
-				error_msg+=" This file is usually provided by sys-fs/{eudev,udev} or sys-apps/systemd!"
+				error_msg+=" This file is usually provided by: sys-apps/systemd-utils, sys-fs/eudev, or sys-apps/systemd!"
 				gen_die "${error_msg}"
 			fi
 
@@ -1065,6 +1101,7 @@ determine_real_args() {
 			FEATURES_REQUIRING_BUSYBOX+=( KEYMAP )
 			FEATURES_REQUIRING_BUSYBOX+=( LVM )
 			FEATURES_REQUIRING_BUSYBOX+=( LUKS )
+			FEATURES_REQUIRING_BUSYBOX+=( KEYCTL )
 			FEATURES_REQUIRING_BUSYBOX+=( MDADM )
 			FEATURES_REQUIRING_BUSYBOX+=( MULTIPATH )
 			FEATURES_REQUIRING_BUSYBOX+=( SPLASH )
@@ -1101,15 +1138,15 @@ determine_real_args() {
 			fi
 		fi
 
-		DU_COMMAND="$(which du 2>/dev/null)"
+		DU_COMMAND="$(type -P du 2>/dev/null)"
 
-		LDDTREE_COMMAND="$(which lddtree 2>/dev/null)"
+		LDDTREE_COMMAND="$(type -P lddtree 2>/dev/null)"
 		if [ -z "${LDDTREE_COMMAND}" ]
 		then
 			gen_die "lddtree not found. Is app-misc/pax-utils installed?"
 		fi
 
-		CPIO_COMMAND="$(which cpio 2>/dev/null)"
+		CPIO_COMMAND="$(type -P cpio 2>/dev/null)"
 		if [[ -z "${CPIO_COMMAND}" ]]
 		then
 			# This will be fatal because we cpio either way
@@ -1154,7 +1191,7 @@ determine_real_args() {
 				gen_die "SANDBOX_ON=1 detected -- You cannot use --sandbox when already running within a sandbox!"
 			fi
 
-			SANDBOX_COMMAND="$(which sandbox 2>/dev/null)"
+			SANDBOX_COMMAND="$(type -P sandbox 2>/dev/null)"
 			if [ -z "${SANDBOX_COMMAND}" ]
 			then
 				gen_die "Sandbox not found. Is sys-apps/sandbox installed?"
@@ -1178,7 +1215,7 @@ determine_real_args() {
 
 	if isTrue "${need_tar}"
 	then
-		TAR_COMMAND="$(which tar 2>/dev/null)"
+		TAR_COMMAND="$(type -P tar 2>/dev/null)"
 		if [ -z "${TAR_COMMAND}" ]
 		then
 			gen_die "tar not found. Is app-arch/tar installed?"
@@ -1193,6 +1230,15 @@ determine_real_args() {
 		fi
 	fi
 
+	KMOD_CMD=$(type -P kmod 2>/dev/null)
+	if ! isTrue "${BUILD_STATIC}"
+	then
+		if [ -z "${KMOD_CMD}" ]
+		then
+			gen_die "kmod not found. Is sys-apps/kmod installed?"
+		fi
+	fi
+
 	if ! isTrue "${CMD_INSTALL}"
 	then
 		if [ -n "${KERNEL_MODULES_PREFIX}" ]
@@ -1241,10 +1287,14 @@ determine_real_args() {
 
 	if isTrue "${FIRMWARE}"
 	then
-		for ff in ${FIRMWARE_FILES}; do
-			[[ ${ff} = /* ]] && gen_die "FIRMWARE_FILES should list paths relative to FIRMWARE_DIR, not absolute."
+		IFS=',' read -r -a FIRMWARE_FILES <<< "${FIRMWARE_FILES}"
+		pushd "${FIRMWARE_DIR}" &>/dev/null || gen_die "Failed to chdir to '${FIRMWARE_DIR}'!"
+		local ff
+		for ff in "${FIRMWARE_FILES[@]}"
+		do
+			[[ ${ff} = /* ]] && gen_die "--firmware-files contains value '${ff}' which is not a relative path from '${FIRMWARE_DIR}'!"
+			[[ ! -e "${ff}" ]] && gen_die "--firmware-files contains value '${ff}' which was not found in '${FIRMWARE_DIR}'!"
 		done
-
-		[[ "${FIRMWARE_FILES}" = *,* ]] && gen_die "FIRMWARE_FILES should be a space-separated list."
+		popd &>/dev/null || gen_die "Failed to chdir!"
 	fi
 }
