@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: 055497398d56ac3f3585dc7514b40b60ad7cb8d1 $
+# $Id: 986b3b4760d83d8029ab887f50e5ff184bd07086 $
 
 COPY_BINARIES=false
 CPIO_ARGS="--quiet --null -o -H newc --owner root:root --force-local"
@@ -37,7 +37,7 @@ copy_binaries() {
 		[[ -e "${binary}" ]] \
 			|| gen_die "Binary ${binary} could not be found"
 
-		if LC_ALL=C "${LDDTREE_COMMAND}" "${binary}" 2>&1 | grep -F -q 'not found'
+		if "${LDDTREE_COMMAND}" "${binary}" 2>&1 | grep -F -q 'not found'
 		then
 			gen_die "Binary ${binary} is linked to missing libraries and may need to be re-built"
 		fi
@@ -484,6 +484,7 @@ append_base_layout() {
 	isTrue "${ZFS}" && build_parameters+=( --zfs ) || build_parameters+=( --no-zfs )
 	isTrue "${SPLASH}" && build_parameters+=( --splash ) || build_parameters+=( --no-splash )
 	isTrue "${STRACE}" && build_parameters+=( --strace ) || build_parameters+=( --no-strace )
+	isTrue "${KEYCTL}" && build_parameters+=( --keyctl ) || build_parameters+=( --no-keyctl )
 	isTrue "${GPG}" && build_parameters+=( --gpg ) || build_parameters+=( --no-gpg )
 	isTrue "${LUKS}" && build_parameters+=( --luks ) || build_parameters+=( --no-luks )
 	isTrue "${FIRMWARE}" && build_parameters+=( --firmware ) || build_parameters+=( --no-firmware )
@@ -562,8 +563,16 @@ append_busybox() {
 	if isTrue "$(is_glibc)"
 	then
 		local libdir=$(get_chost_libdir)
-		mkdir -p "${TDIR}"/lib || gen_die "Failed to create '${TDIR}/lib'!"
-		copy_system_binaries "${TDIR}"/lib "${libdir}"/libnss_dns.so
+		local libnss_dns="${libdir}/libnss_dns.so"
+
+		# NSS dns module was moved into libc in >=glibc-2.34
+		# but when this file exists we are probably dealing with older glibc
+		# and need to manually copy the module.
+		if [[ -f "${libnss_dns}" ]]
+		then
+			mkdir -p "${TDIR}"/lib || gen_die "Failed to create '${TDIR}/lib'!"
+			copy_system_binaries "${TDIR}"/lib "${libnss_dns}"
+		fi
 	fi
 
 	log_future_cpio_content
@@ -628,7 +637,8 @@ append_eudev() {
 		local gen_hwdb_cmd=( "${TDIR}/usr/bin/udevadm" )
 		gen_hwdb_cmd+=( hwdb --update --root "${TDIR}" )
 		print_info 3 "COMMAND: ${gen_hwdb_cmd[*]}" 1 0 1
-		eval "${gen_hwdb_cmd[@]}" || gen_die "Failed to pre-generate initramfs' /etc/udev/hwdb.bin!"
+		eval "${gen_hwdb_cmd[@]}" 2>&1 | tee -a "${LOGFILE}" \
+			|| gen_die "Failed to pre-generate initramfs' /etc/udev/hwdb.bin!"
 
 		# Now that we have a pre-generated hwdb in initramfs
 		# we can delete source files
@@ -791,7 +801,7 @@ append_multipath() {
 
 	local udevdir=$(get_udevdir)
 	local udevdir_initramfs="/usr/lib/udev"
-	local udev_files=( $(qlist -e sys-fs/multipath-tools:0 \
+	local udev_files=( $(qlist -eC sys-fs/multipath-tools:0 \
 		| grep -E -- "^${udevdir}")
 	)
 
@@ -892,6 +902,33 @@ append_iscsi() {
 	fi
 }
 
+append_keyutils() {
+	local PN=keyutils
+	local TDIR="${TEMP}/initramfs-${PN}-temp"
+	if [ -d "${TDIR}" ]
+	then
+		rm -r "${TDIR}" || gen_die "Failed to clean out existing '${TDIR}'!"
+	fi
+
+	populate_binpkg ${PN}
+
+	mkdir "${TDIR}" || gen_die "Failed to create '${TDIR}'!"
+
+	unpack "$(get_gkpkg_binpkg "${PN}")" "${TDIR}"
+
+	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
+
+	log_future_cpio_content
+	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
+		|| gen_die "Failed to append ${PN} to cpio!"
+
+	cd "${TEMP}" || die "Failed to chdir to '${TEMP}'!"
+	if isTrue "${CLEANUP}"
+	then
+		rm -rf "${TDIR}"
+	fi
+}
+
 append_lvm() {
 	local PN=lvm
 	local TDIR="${TEMP}/initramfs-${PN}-temp"
@@ -1066,7 +1103,8 @@ append_zfs() {
 	mkdir -p "${TDIR}"/etc/zfs || gen_die "Failed to create '${TDIR}/etc/zfs'!"
 
 	# Copy files to /etc/zfs
-	for i in zdev.conf zpool.cache
+	local i
+	for i in vdev_id.conf zdev.conf zpool.cache
 	do
 		if [ -f /etc/zfs/${i} ]
 		then
@@ -1093,7 +1131,7 @@ append_zfs() {
 
 	local udevdir=$(get_udevdir)
 	local udevdir_initramfs="/usr/lib/udev"
-	local udev_files=( $(qlist -e sys-fs/zfs:0 \
+	local udev_files=( $(qlist -eC sys-fs/zfs:0 \
 		| grep -E -- "^${udevdir}")
 	)
 
@@ -1110,6 +1148,15 @@ append_zfs() {
 		local dest_file="${TDIR%/}${udev_file/${udevdir}/${udevdir_initramfs}}"
 		cp -aL "${udev_file}" "${dest_file}" \
 			|| gen_die "Failed to copy '${udev_file}' to '${dest_file}'"
+
+		if [[ "${dest_file}" == *.rules ]]
+		then
+			print_info 5 "Updating UDEV dir in '${dest_file}' ..."
+			sed -i \
+				-e "s|${udevdir}|${udevdir_initramfs}|g" \
+				"${dest_file}" \
+				|| gen_die "Failed to update UDEV dir in '${dest_file}'"
+		fi
 	done
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
@@ -1206,7 +1253,7 @@ append_linker() {
 		mkdir -p "${TDIR}"/sbin || gen_die "Failed to create '${TDIR}/sbin'!"
 
 		local libdir=$(get_chost_libdir)
-		copy_system_binaries "${TDIR}/sbin" "${libdir}/../../sbin/ldconfig"
+		copy_system_binaries "${TDIR}/sbin" "${libdir%/usr\/*}/sbin/ldconfig"
 	else
 		# Only copy /etc/ld.so.conf.d -- /etc/ld.so.conf was already
 		# added to CPIO via append_base_layout() and because we only
@@ -1582,8 +1629,16 @@ append_dropbear() {
 	if isTrue "$(is_glibc)"
 	then
 		local libdir=$(get_chost_libdir)
-		mkdir -p "${TDIR}"/lib || gen_die "Failed to create '${TDIR}/lib'!"
-		copy_system_binaries "${TDIR}"/lib "${libdir}"/libnss_files.so
+		local libnss_files="${libdir}/libnss_files.so"
+
+		# NSS files module was moved into libc in >=glibc-2.34
+		# but when this file exists we are probably dealing with older glibc
+		# and need to manually copy the module.
+		if [[ -f "${libnss_files}" ]]
+		then
+			mkdir -p "${TDIR}"/lib || gen_die "Failed to create '${TDIR}/lib'!"
+			copy_system_binaries "${TDIR}"/lib "${libnss_files}"
+		fi
 	fi
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
@@ -1651,10 +1706,10 @@ append_firmware() {
 
 	mkdir -p "${TDIR}"/lib/firmware || gen_die "Failed to create '${TDIR}/lib/firmware'!"
 
-	if [ -n "${FIRMWARE_FILES}" ]
+	if [ ${#FIRMWARE_FILES[@]} -gt 0 ]
 	then
 		pushd "${FIRMWARE_DIR}" &>/dev/null || gen_die "Failed to chdir to '${FIRMWARE_DIR}'!"
-		cp -rL --parents --target-directory="${TDIR}/lib/firmware" ${FIRMWARE_FILES} 2>/dev/null \
+		cp -rL --parents --target-directory="${TDIR}/lib/firmware" "${FIRMWARE_FILES[@]}" 2>/dev/null \
 			|| gen_die "Failed to copy firmware files (${FIRMWARE_FILES}) to '${TDIR}/lib/firmware'!"
 		popd &>/dev/null || gen_die "Failed to chdir!"
 	else
@@ -1716,9 +1771,19 @@ append_modules() {
 		rm -r "${TDIR}" || gen_die "Failed to clean out existing '${TDIR}'!"
 	fi
 
+	populate_binpkg kmod
+
 	mkdir "${TDIR}" || gen_die "Failed to create '${TDIR}'!"
+
+	unpack "$(get_gkpkg_binpkg kmod)" "${TDIR}"
+
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
 
+	# Delete unneeded files
+	rm -rf \
+		usr/include \
+		usr/lib
+
 	local mydir=
 	for mydir in \
 		etc/modules \
@@ -1743,23 +1808,24 @@ append_modules() {
 		gen_die "${error_message}"
 	fi
 
+	determine_KEXT
+
 	cd "${modules_srcdir}" || gen_die "Failed to chdir to '${modules_srcdir}'!"
 
 	print_info 2 "$(get_indent 2)modules: >> Copying modules from '${modules_srcdir}' to initramfs ..."
 
 	local i= mymod=
-	local MOD_EXT="$(modules_kext)"
 	local n_copied_modules=0
 	for i in $(gen_dep_list)
 	do
-		mymod=$(find . -name "${i}${MOD_EXT}" 2>/dev/null | head -n 1)
+		mymod=$(find . -name "${i}${KEXT}" 2>/dev/null | head -n 1)
 		if [ -z "${mymod}" ]
 		then
-			print_warning 3 "$(get_indent 3) - ${i}${MOD_EXT} not found; Skipping ..."
+			print_warning 3 "$(get_indent 3) - ${i}${KEXT} not found; Skipping ..."
 			continue;
 		fi
 
-		print_info 3 "$(get_indent 3) - Copying ${i}${MOD_EXT} ..."
+		print_info 3 "$(get_indent 3) - Copying ${i}${KEXT} ..."
 		cp -ax --parents --target-directory "${modules_dstdir}" "${mymod}" 2>/dev/null \
 			|| gen_die "Failed to copy '${modules_srcdir}/${mymod}' to '${modules_dstdir}'!"
 		n_copied_modules=$[$n_copied_modules+1]
@@ -1776,7 +1842,7 @@ append_modules() {
 		|| gen_die "Failed to copy '${modules_srcdir}/modules*' to '${modules_dstdir}'!"
 
 	print_info 2 "$(get_indent 2)modules: Updating modules.dep ..."
-	local a depmod_cmd=( depmod -a -b "${TDIR}" ${KV} )
+	local depmod_cmd=( depmod -a -b "${TDIR}" ${KV} )
 	print_info 3 "COMMAND: ${depmod_cmd[*]}" 1 0 1
 	eval "${depmod_cmd[@]}" || gen_die "Failed to run '${depmod_cmd[*]}'!"
 
@@ -1810,10 +1876,19 @@ append_modprobed() {
 	mkdir "${TDIR}" || gen_die "Failed to create '${TDIR}'!"
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
 
-	mkdir -p "${TDIR}"/etc || gen_die "Failed to create '${TDIR}/etc'!"
+	local modprobe_dir
+	for modprobe_dir in /etc/modprobe.d /lib/modprobe.d
+	do
+		if [[ ! -e "${modprobe_dir}" ]]; then
+			print_info 5 "'${modprobe_dir}' does not exist; Skipping ..."
+			continue
+		fi
 
-	cp -rL "/etc/modprobe.d" "${TDIR}"/etc/ 2>/dev/null \
-		|| gen_die "Failed to copy '/etc/modprobe.d'!"
+		mkdir -p "${TDIR}${modprobe_dir}" || gen_die "Failed to create '${TDIR}${modprobe_dir}'!"
+
+		cp -aL "${modprobe_dir}"/. "${TDIR}${modprobe_dir}" 2>/dev/null \
+			|| gen_die "Failed to copy '${modprobe_dir}'!"
+	done
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
 	log_future_cpio_content
@@ -1827,13 +1902,7 @@ append_modprobed() {
 	fi
 }
 
-# check for static linked file with objdump
-is_static() {
-	LANG="C" LC_ALL="C" objdump -T $1 2>&1 | grep "not a dynamic object" > /dev/null
-	return $?
-}
-
-append_auxilary() {
+append_auxiliary() {
 	local TDIR="${TEMP}/initramfs-aux-temp"
 	if [ -d "${TDIR}" ]
 	then
@@ -1956,7 +2025,7 @@ append_auxilary() {
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
 	log_future_cpio_content
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
-		|| gen_die "Failed to append auxilary to cpio!"
+		|| gen_die "Failed to append auxiliary to cpio!"
 
 	cd "${TEMP}" || die "Failed to chdir to '${TEMP}'!"
 	if isTrue "${CLEANUP}"
@@ -1966,11 +2035,14 @@ append_auxilary() {
 }
 
 append_data() {
-	local name=$1 var=$2
+	[ $# -eq 0 ] && gen_die "append_data() called with zero arguments"
+
+	local name=$1
 	local func="append_${name}"
 
-	[ $# -eq 0 ] && gen_die "append_data() called with zero arguments"
-	if [ $# -eq 1 ] || isTrue "${var}"
+	shift
+
+	if [ $# -eq 0 ] || anyTrue "$@"
 	then
 		print_info 1 "$(get_indent 1)>> Appending ${name} cpio data ..."
 		${func} || gen_die "${func}() failed!"
@@ -1980,14 +2052,24 @@ append_data() {
 create_initramfs() {
 	print_info 1 "initramfs: >> Initializing ..."
 
+	if ! isTrue "${BUILD_KERNEL}"
+	then
+		# Check early for suitable kmod
+		determine_KEXT
+		if ! isTrue "$(is_kext_supported_by_kmod "${KEXT}")"
+		then
+			gen_die "${KMOD_CMD} does not support chosen module compression algorithm. Please re-emerge sys-apps/kmod with USE=$(get_kext_kmod_use_flag "${KEXT}") enabled or adjust CONFIG_MODULE_COMPRESS_* kernel option!"
+		fi
+	fi
+
 	# Create empty cpio
 	CPIO_ARCHIVE="${TMPDIR}/${GK_FILENAME_TEMP_INITRAMFS}"
 	append_data 'devices' # WARNING, must be first!
 	append_data 'base_layout'
 	append_data 'util-linux'
 	append_data 'eudev'
-	append_data 'devicemanager'
-	append_data 'auxilary' "${BUSYBOX}"
+	append_data 'devicemanager' "${DMRAID}" "${LVM}" "${LUKS}" "${MULTIPATH}"
+	append_data 'auxiliary' "${BUSYBOX}"
 	append_data 'busybox' "${BUSYBOX}"
 	append_data 'b2sum' "${B2SUM}"
 	append_data 'btrfs' "${BTRFS}"
@@ -1996,6 +2078,7 @@ create_initramfs() {
 	append_data 'e2fsprogs' "${E2FSPROGS}"
 	append_data 'gpg' "${GPG}"
 	append_data 'iscsi' "${ISCSI}"
+	append_data 'keyutils' "${KEYCTL}"
 	append_data 'luks' "${LUKS}"
 	append_data 'lvm' "${LVM}"
 	append_data 'bcache' "${BCACHE}"
