@@ -1,10 +1,10 @@
 #!/bin/bash
-# $Id: 5e6c980968fa01ab920692966c28074489277451 $
+# $Id: 89a562b79ab0d91504ba4d41e89edce38cc723fe $
 
 gen_dep_list() {
 	if isTrue "${ALLRAMDISKMODULES}"
 	then
-		strip_mod_paths $(find "${INSTALL_MOD_PATH}/lib/modules/${KV}" -name "*$(modules_kext)") | sort
+		strip_mod_paths $(find "${KERNEL_MODULES_PREFIX%/}/lib/modules/${KV}" -name "*${KEXT}") | sort
 	else
 		rm -f "${TEMP}/moddeps" >/dev/null
 
@@ -47,25 +47,12 @@ gen_deps() {
 }
 
 modules_dep_list() {
-	KEXT=$(modules_kext)
-	if [ -f ${INSTALL_MOD_PATH}/lib/modules/${KV}/modules.dep ]
+	if [ -f "${KERNEL_MODULES_PREFIX%/}/lib/modules/${KV}/modules.dep" ]
 	then
-		cat ${INSTALL_MOD_PATH}/lib/modules/${KV}/modules.dep | grep ${1}${KEXT}\: | cut -d\:  -f2
+		grep -F -- "/${1}${KEXT}:" "${KERNEL_MODULES_PREFIX%/}/lib/modules/${KV}/modules.dep" | cut -d\:  -f2
 	fi
 }
 
-modules_kext() {
-	local KEXT='.ko'
-
-	if grep -sq '^CONFIG_MODULE_COMPRESS=y' "${KERNEL_OUTPUTDIR}"/.config
-	then
-		grep -sq '^CONFIG_MODULE_COMPRESS_XZ=y' "${KERNEL_OUTPUTDIR}"/.config && KEXT='.ko.xz'
-		grep -sq '^CONFIG_MODULE_COMPRESS_GZIP=y' "${KERNEL_OUTPUTDIR}"/.config && KEXT='.ko.gz'
-	fi
-
-	echo ${KEXT}
-}
-
 # Pass module deps list
 strip_mod_paths() {
 	local x
